/*
Copyright The Kubernetes Authors.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
*/

// Code generated by client-gen. DO NOT EDIT.

package v1

import (
	"context"
	"time"

	v1 "github.com/nokia/danm/crd/apis/danm/v1"
	scheme "github.com/nokia/danm/crd/client/clientset/versioned/scheme"
	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
	types "k8s.io/apimachinery/pkg/types"
	watch "k8s.io/apimachinery/pkg/watch"
	rest "k8s.io/client-go/rest"
)

// TenantNetworksGetter has a method to return a TenantNetworkInterface.
// A group's client should implement this interface.
type TenantNetworksGetter interface {
	TenantNetworks(namespace string) TenantNetworkInterface
}

// TenantNetworkInterface has methods to work with TenantNetwork resources.
type TenantNetworkInterface interface {
	Create(ctx context.Context, tenantNetwork *v1.TenantNetwork, opts metav1.CreateOptions) (*v1.TenantNetwork, error)
	Update(ctx context.Context, tenantNetwork *v1.TenantNetwork, opts metav1.UpdateOptions) (*v1.TenantNetwork, error)
	Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error
	DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error
	Get(ctx context.Context, name string, opts metav1.GetOptions) (*v1.TenantNetwork, error)
	List(ctx context.Context, opts metav1.ListOptions) (*v1.TenantNetworkList, error)
	Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error)
	Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.TenantNetwork, err error)
	TenantNetworkExpansion
}

// tenantNetworks implements TenantNetworkInterface
type tenantNetworks struct {
	client rest.Interface
	ns     string
}

// newTenantNetworks returns a TenantNetworks
func newTenantNetworks(c *DanmV1Client, namespace string) *tenantNetworks {
	return &tenantNetworks{
		client: c.RESTClient(),
		ns:     namespace,
	}
}

// Get takes name of the tenantNetwork, and returns the corresponding tenantNetwork object, and an error if there is any.
func (c *tenantNetworks) Get(ctx context.Context, name string, options metav1.GetOptions) (result *v1.TenantNetwork, err error) {
	result = &v1.TenantNetwork{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("tenantnetworks").
		Name(name).
		VersionedParams(&options, scheme.ParameterCodec).
		Do(ctx).
		Into(result)
	return
}

// List takes label and field selectors, and returns the list of TenantNetworks that match those selectors.
func (c *tenantNetworks) List(ctx context.Context, opts metav1.ListOptions) (result *v1.TenantNetworkList, err error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	result = &v1.TenantNetworkList{}
	err = c.client.Get().
		Namespace(c.ns).
		Resource("tenantnetworks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Do(ctx).
		Into(result)
	return
}

// Watch returns a watch.Interface that watches the requested tenantNetworks.
func (c *tenantNetworks) Watch(ctx context.Context, opts metav1.ListOptions) (watch.Interface, error) {
	var timeout time.Duration
	if opts.TimeoutSeconds != nil {
		timeout = time.Duration(*opts.TimeoutSeconds) * time.Second
	}
	opts.Watch = true
	return c.client.Get().
		Namespace(c.ns).
		Resource("tenantnetworks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Timeout(timeout).
		Watch(ctx)
}

// Create takes the representation of a tenantNetwork and creates it.  Returns the server's representation of the tenantNetwork, and an error, if there is any.
func (c *tenantNetworks) Create(ctx context.Context, tenantNetwork *v1.TenantNetwork, opts metav1.CreateOptions) (result *v1.TenantNetwork, err error) {
	result = &v1.TenantNetwork{}
	err = c.client.Post().
		Namespace(c.ns).
		Resource("tenantnetworks").
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(tenantNetwork).
		Do(ctx).
		Into(result)
	return
}

// Update takes the representation of a tenantNetwork and updates it. Returns the server's representation of the tenantNetwork, and an error, if there is any.
func (c *tenantNetworks) Update(ctx context.Context, tenantNetwork *v1.TenantNetwork, opts metav1.UpdateOptions) (result *v1.TenantNetwork, err error) {
	result = &v1.TenantNetwork{}
	err = c.client.Put().
		Namespace(c.ns).
		Resource("tenantnetworks").
		Name(tenantNetwork.Name).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(tenantNetwork).
		Do(ctx).
		Into(result)
	return
}

// Delete takes name of the tenantNetwork and deletes it. Returns an error if one occurs.
func (c *tenantNetworks) Delete(ctx context.Context, name string, opts metav1.DeleteOptions) error {
	return c.client.Delete().
		Namespace(c.ns).
		Resource("tenantnetworks").
		Name(name).
		Body(&opts).
		Do(ctx).
		Error()
}

// DeleteCollection deletes a collection of objects.
func (c *tenantNetworks) DeleteCollection(ctx context.Context, opts metav1.DeleteOptions, listOpts metav1.ListOptions) error {
	var timeout time.Duration
	if listOpts.TimeoutSeconds != nil {
		timeout = time.Duration(*listOpts.TimeoutSeconds) * time.Second
	}
	return c.client.Delete().
		Namespace(c.ns).
		Resource("tenantnetworks").
		VersionedParams(&listOpts, scheme.ParameterCodec).
		Timeout(timeout).
		Body(&opts).
		Do(ctx).
		Error()
}

// Patch applies the patch and returns the patched tenantNetwork.
func (c *tenantNetworks) Patch(ctx context.Context, name string, pt types.PatchType, data []byte, opts metav1.PatchOptions, subresources ...string) (result *v1.TenantNetwork, err error) {
	result = &v1.TenantNetwork{}
	err = c.client.Patch(pt).
		Namespace(c.ns).
		Resource("tenantnetworks").
		Name(name).
		SubResource(subresources...).
		VersionedParams(&opts, scheme.ParameterCodec).
		Body(data).
		Do(ctx).
		Into(result)
	return
}
